@@ -215,108 +215,6 @@ sphcoord <- function(ccoord){
   c(theta,phi)
 }
 
-design.spheven <- function(order,gradients,lambda){
-#
-#  compute design matrix for Q-ball
-#
-  order <- as.integer(max(0,order))
-  if(order%%2==1){
-    warning("maximum order needs to be even, increase order by one")
-    order <- order+1
-  } 
-
-  # calculate spherical angles theta and phi corresponding to the gradients
-  n <- dim(gradients)[2]
-  theta <- phi <- numeric(n)
-  for( i in 1:n){
-    angles <- sphcoord(gradients[,i])
-    theta[i] <- angles[1]
-    phi[i] <-  angles[2]
-  }
-
-  # values of SH on specified spherical angles
-  sphharmonics <- getsphericalharmonicseven(order,theta,phi)
-  # Laplace-Beltrami-Regularization term
-  lord <- rep(seq(0,order,2),2*seq(0,order,2)+1)
-  L <- lambda*diag(lord^2*(lord+1)^2)
-  # transformation matrix for SH coefficients
-  ttt <- solve(sphharmonics%*%t(sphharmonics)+L)%*%sphharmonics
-  # results
-  list(design = sphharmonics,
-       matrix = ttt,
-       theta = theta,
-       phi = phi)
-}
-design.wspheven <- function(order,gradients,lambda){
-#
-#  compute design matrix for Q-ball 
-#  Regularization differs from q ball (Desceoutoux, Tuch)
-#
-  order <- as.integer(max(0,order))
-  if(order%%2==1){
-    warning("maximum order needs to be even, increase order by one")
-    order <- order+1
-  } 
-
-  # calculate spherical angles theta and phi corresponding to the gradients
-  n <- dim(gradients)[2]
-  theta <- phi <- numeric(n)
-  for( i in 1:n){
-    angles <- sphcoord(gradients[,i])
-    theta[i] <- angles[1]
-    phi[i] <-  angles[2]
-  }
-
-  # values of SH on specified spherical angles
-  sphharmonics <- getsphericalharmonicseven(order,theta,phi)
-  # Laplace-Beltrami-Regularization term
-  lord <- rep(seq(0,order,2),2*seq(0,order,2)+1)
-  L <- lambda*diag(lord^3*(lord+1)^3)
-  # transformation matrix for SH coefficients
-  ttt <- solve(sphharmonics%*%t(sphharmonics)+L)%*%sphharmonics
-  # results
-  list(design = sphharmonics,
-       matrix = ttt,
-       theta = theta,
-       phi = phi)
-}
-
-design.sphall <- function(order,gradients,lambda){
-#
-#  compute design matrix for Q-ball
-#
-  order <- as.integer(max(0,order))
-
-  # calculate spherical angles theta and phi corresponding to the gradients
-  n <- dim(gradients)[2]
-  theta <- phi <- numeric(n)
-  for( i in 1:n){
-    angles <- sphcoord(gradients[,i])
-    theta[i] <- angles[1]
-    phi[i] <-  angles[2]
-  }
-
-  # values of SH on specified spherical angles
-  sphharmonics <- getsphericalharmonicsall(order,theta,phi)
-  # Laplace-Beltrami-Regularization term
-  lord <- rep(0:order,2*(0:order)+1)
-  L <- lambda*diag(lord^2*(lord+1)^2)
-  # transformation matrix for SH coefficients
-  ttt <- solve(sphharmonics%*%t(sphharmonics)+L)%*%sphharmonics
-
-  # results
-  list(design = sphharmonics,
-       matrix = ttt,
-       theta = theta,
-       phi = phi)
-}
-
-plzero <- function(order){
-  l <- seq(2,order,2)
-  pl <- l
-  for(i in 1:length(l)) pl[i] <- (-1)^(l[i]/2)*prod(seq(1,(l[i]-1),2))/prod(seq(2,l[i],2))
-  2*pi*diag(rep(c(1,pl),2*seq(0,order,2)+1))
-}
 
 gettriangles <- function(gradients){
   dgrad <- dim(gradients)
