@@ -100,12 +100,12 @@ C includes regularization of D
          relrss = (oldrss-rss)/rss
          IF(maxabsdg.lt.eps.or.relrss.lt.1d-6) THEN
 C  prepare things for return if gradient is close to 0
-            call regD0(D,negdefin)
+            call regularD(D,negdefin)
             RETURN
          END IF
          IF(iter.gt.1.and.abs(rho(1)*rho(4)*rho(6)).lt.1.d-10) THEN
 C  prepare things for return if gradient is close to 0
-            call regD0(D,negdefin)
+            call regularD(D,negdefin)
             RETURN
          END IF
          gamma=min(gamma/alpha,1.d0)
@@ -149,7 +149,7 @@ C  next iteration
                END DO
                th0n=th0-gamma*pk(7)
                nrss=0.d0
-               call rho2D(rhon,Dn)
+               call rho2D0(rhon,Dn)
                DO i=1,nb
                   z=b(1,i)*Dn(1)
                   DO j=2,6
@@ -182,7 +182,7 @@ C  decrease gamma and try new regularization
          rss=nrss
          call rchkusr()
       END DO
-      call regD0(D,negdefin)
+      call regularD(D,negdefin)
       RETURN
       END
       subroutine dslvdtir(s,nb,b,varinv,th0,D,F,niter,eps,rss)
@@ -255,12 +255,12 @@ C
          relrss = (oldrss-rss)/rss
          IF(maxabsdg.lt.eps.or.relrss.lt.1d-6) THEN
 C  prepare things for return if gradient is close to 0
-            call regD0(D,negdefin)
+            call regularD(D,negdefin)
             RETURN
          END IF
          IF(iter.gt.1.and.abs(rho(1)*rho(4)*rho(6)).lt.1.d-10) THEN
 C  prepare things for return if gradient is close to 0
-            call regD0(D,negdefin)
+            call regularD(D,negdefin)
             RETURN
          END IF
          gamma=min(gamma/alpha,1.d0)
@@ -303,7 +303,7 @@ C  next iteration
                END DO
                th0n=th0-gamma*pk(7)
                nrss=0.d0
-               call rho2D(rhon,Dn)
+               call rho2D0(rhon,Dn)
                DO i=1,nb
                   z=b(1,i)*Dn(1)
                   DO j=2,6
@@ -336,6 +336,73 @@ C  decrease gamma and try new regularization
          rss=nrss
          call rchkusr()
       END DO
-      call regD0(D,negdefin)
+      call regularD(D,negdefin)
+      RETURN
+      END
+CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
+C
+C     get D from rho without regularization
+C
+CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
+      subroutine rho2D0(rho,D)
+      implicit logical(a-z)
+      real*8 D(6),rho(6),r1,r2,r3,r4,r5,r6
+      r1=rho(1)
+      r2=rho(2)
+      r3=rho(3)
+      r4=rho(4)
+      r5=rho(5)
+      r6=rho(6)
+      D(1)=r1*r1
+      D(2)=r1*r2
+      D(3)=r1*r3
+      D(4)=r2*r2+r4*r4
+      D(5)=r2*r3+r4*r5
+      D(6)=r3*r3+r5*r5+r6*r6
+      RETURN
+      END
+C
+C
+C   compute Least Squares Criterion based on positive definit tensor parametrization D = R^T R
+C
+C     
+      subroutine opttensR(par,si,s0,grad,ngrad,sdcoef,erg)
+      implicit logical (a-z)
+      integer ngrad
+      real*8 par(6),si(ngrad),s0,grad(3,ngrad),sdcoef(4),erg
+      integer i
+      real*8 low,up,g1,g2,g3,s,D(6),z,w
+      low=sdcoef(1)+sdcoef(3)*sdcoef(2)
+      up=sdcoef(1)+sdcoef(4)*sdcoef(2)
+      call rho2D0(par,D)
+      s=0.d0
+      DO i = 1,ngrad
+         g1 = grad(1,i)
+         g2 = grad(2,i)
+         g3 = grad(3,i)
+         z =  g1*g1*D(1)+g2*g2*D(4)+g3*g3*D(6)+
+     1        2.d0*(g1*g2*D(2)+g1*g3*D(3)+g2*g3*D(5))
+         w = max(low,(min(up,sdcoef(1)+si(i)*sdcoef(2))))
+         z = (si(i)-s0*exp(-z))/w
+         s = s+z*z
+      END DO
+      erg = s
+      RETURN
+      END      
+      subroutine tensRres(par,si,s0,grad,ngrad,res)
+      implicit logical (a-z)
+      integer ngrad
+      real*8 par(6),si(ngrad),s0,grad(3,ngrad)
+      integer i
+      real*8 g1,g2,g3,D(6),res(ngrad),z
+      call rho2D0(par,D)
+      DO i = 1,ngrad
+         g1 = grad(1,i)
+         g2 = grad(2,i)
+         g3 = grad(3,i)
+         z =  g1*g1*D(1)+g2*g2*D(4)+g3*g3*D(6)+
+     1        2.d0*(g1*g2*D(2)+g1*g3*D(3)+g2*g3*D(5))
+         res(i) = (si(i)-s0*exp(-z))
+      END DO
       RETURN
       END
